/**                                                                                         
 * @file egpc.c
 *
 * Program for reading/writing/configuring gpio character devices.
 * Code here based upon the egp, the gpio program for the SoM-5282M/SoM-100ES.
 * See the usage message for infomation on how to use this application.
 *
 * @author EMAC, Inc <support@emacinc.com>
 * 
 * Copyright (C) 2009, EMAC, Inc
 */
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stddef.h>
#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <asm/types.h>
#include "gpio_char.h"

#define ARGCHECK(num){if(argc<num){printf("invalid number of arguements\n");help();return -1;}}
#define OPENDEV(filed,dev){if((filed = open(dev, O_RDWR))<=0){printf("couldn't open %s\n",dev);return -1;}}

static int help(void)
{
    const char *help_table = 
        "**********************************************\n"
        "egpc                                          \n"
        "EMAC.Inc                                      \n"
        ""__DATE__"                                    \n"
        "usage:                                        \n"
        "egpc device d/c/i (writearg)                  \n"
        "commands                                      \n"
        "                                              \n"
        "configure      c  (mask)                      \n"
        "configures the pins for input/output or returns\n"
        "the current setting if no mask is specified   \n"
        "mask is input/output bitmask                  \n"
        "1 = output 0 = input                          \n"
        "ex:                                           \n"
        "egp /dev/egpio0  c 0xaa                       \n"
        "sets every other pin to be an output          \n"
        "egp /dev/egpio0  c                            \n"
        "reads the current pin configuration state     \n"
        "                                              \n"
        "data      d (data)                            \n"
        "sets the ouput latch high/low according       \n"
        "to the data bitmask.                          \n"
        "If the data bitmask is not specified, the current\n"
        "pin state is retrieved.                       \n"
        "ex:                                           \n"
        "egp /dev/egpio1  d 0xff                       \n"
        "writes the ouput latch all high               \n"
        "egp /dev/egpio1  d                            \n"
        "reads the current pin state                   \n"
        "                                              \n"
        "index      i (index)                          \n"
        "sets the index                                \n"
        "If the index is not specified, the current    \n"
        "index is retrieved.                           \n"
        "ex:                                           \n"
        "egp /dev/egpio1  d 0xff                       \n"
        "writes the ouput latch all high               \n"
        "egp /dev/egpio1  d                            \n"
        "reads the current pin state                   \n"
        "**********************************************\n";
    printf("%s", help_table);
    return 0;
}

int main(int argc, char **argv)
{
    int fd;
    __u32 arg[1];
    int command;
    ARGCHECK(3);
    OPENDEV(fd,argv[1]);
    
    switch (*argv[2])
    //determine the correct ioctl command
    {
    case 'c':
        command = (argc<4 ) ? DDRREAD : DDRWRITE;
        break;
    case 'd':
        command = (argc<4 ) ? DATAREAD : DATAWRITE;
        break;
    case 'i':
        command = (argc<4 ) ? INDEXREAD : INDEXWRITE;
        break;
    default:
        printf("invalid command - %s\n", argv[2]);
        help();
        return -1;
    }

    if (argc>3)//get data/mask/index if available
        *arg = strtoul(argv[3], NULL, 0);

    if (ioctl(fd, command, arg)<0) {//perform the operation
        printf("operation not supported by this device\n");
        return -1;
    }

    if (argc<4)
        printf("0x%x\n", *arg);//print return value for reads

    return 0;
}

