/**
 * @file watchdog-test.c
 *
 * This is an example test for the Linux watchdog API based strongly
 * on the watchdog test applications in the Linux kernel documentation
 * (see the docs directory under this project). The default watchdog
 * used on the SoM-9260M is the Softdog (Software Watchdog) module
 * in the Linux Kernel. Although the AT91SAM9260 processor has a built-in
 * watchdog, it has several characteristics which make it difficult to use
 * in an operating system environment (i.e. one-time programmable, enabled
 * by default, ...).
 *
 * @author EMAC, Inc. <support@emacinc.com> (based on Linux kernel watchdog-test.c)
 *
 */
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
 * Watchdog Driver Test Program
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/watchdog.h>

int fd;

/*
 * This function simply sends an IOCTL to the driver, which in turn ticks
 * the watchdog to reset its internal timer so it doesn't trigger a computer reset.
 */
void keep_alive(void)
{
	int dummy;

	ioctl(fd, WDIOC_KEEPALIVE, &dummy);
}

/*
 * The main program.  Run the program with "-d" to disable the card,
 * or "-e" to enable the card or "-t" to set the timeout.
 */
int main(int argc, char *argv[])
{
	fd = open("/dev/watchdog", O_WRONLY);

	if (fd == -1) {
		fprintf(stderr, "Watchdog device not enabled.\n");
		fflush(stderr);
		exit(-1);
	}

	if (argc > 1) {
		if (!strncasecmp(argv[1], "-d", 2)) {
			ioctl(fd, WDIOC_SETOPTIONS, WDIOS_DISABLECARD);
			fprintf(stderr, "Watchdog card disabled.\n");
			fflush(stderr);
			exit(0);
		} else if (!strncasecmp(argv[1], "-e", 2)) {
			ioctl(fd, WDIOC_SETOPTIONS, WDIOS_ENABLECARD);
			fprintf(stderr, "Watchdog card enabled.\n");
			fflush(stderr);
			exit(0);
		} else if (!strncasecmp(argv[1], "-t", 2)) {
			int timeout = 5; /* 5 seconds */
			ioctl(fd, WDIOC_SETTIMEOUT, &timeout);
			fprintf(stderr, "Watchdog timeout set to %d seconds!\n", timeout);
			fflush(stderr);
			exit(0);
		} else {
			fprintf(stderr, "-d to disable, -e to enable.\n");
			fprintf(stderr, "run by itself to tick the card.\n");
			fflush(stderr);
			exit(0);
		}
	} else {
		fprintf(stderr, "Watchdog Ticking Away!\n");
		fflush(stderr);
	}

	while(1) {
		keep_alive();
		sleep(1);
	}
}
